@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2020 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -469,7 +469,7 @@ ModuleExport size_t RegisterTIMImage(void)
   entry=SetMagickInfo("TIM");
   entry->decoder=(DecodeImageHandler *) ReadTIMImage;
   entry->description=ConstantString("PSX TIM");
-  entry->module=ConstantString("TIM");
+  entry->magick_module=ConstantString("TIM");
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
 }
