@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2020 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -265,7 +265,7 @@ MagickExport MagickBooleanType AnnotateImage(Image *image,
   for (p=text; *p != '\0'; p++)
     if (*p == '\n')
       number_lines++;
-  textlist=AcquireQuantumMemory(number_lines+1,sizeof(*textlist));
+  textlist=(char **) AcquireQuantumMemory(number_lines+1,sizeof(*textlist));
   if (textlist == (char **) NULL)
     {
       annotate_info=DestroyDrawInfo(annotate_info);
@@ -274,7 +274,7 @@ MagickExport MagickBooleanType AnnotateImage(Image *image,
       return(MagickFalse);
     }
   p=text;
-  for (i=0; i < number_lines; i++)
+  for (i=0; i < (ssize_t) number_lines; i++)
   {
     char
       *q;
@@ -324,8 +324,10 @@ MagickExport MagickBooleanType AnnotateImage(Image *image,
     (void) CloneString(&annotate->text,textlist[i]);
     if ((metrics.width == 0) || (annotate->gravity != NorthWestGravity))
       (void) GetTypeMetrics(image,annotate,&metrics);
-    height=(ssize_t) (metrics.ascent-metrics.descent+
-      draw_info->interline_spacing+0.5);
+    height=(size_t) ceil(metrics.ascent-metrics.descent+
+      draw_info->interline_spacing-0.5);
+    if (height == 0)
+      height=(size_t) draw_info->pointsize;
     switch (annotate->gravity)
     {
       case UndefinedGravity:
@@ -699,15 +701,24 @@ MagickExport MagickBooleanType GetMultilineTypeMetrics(Image *image,
   char
     **textlist;
 
+  double
+    height;
+
   DrawInfo
     *annotate_info;
 
   MagickBooleanType
     status;
 
+  MagickSizeType
+    size;
+
   register ssize_t
     i;
 
+  size_t
+    count;
+
   TypeMetric
     extent;
 
@@ -725,7 +736,7 @@ MagickExport MagickBooleanType GetMultilineTypeMetrics(Image *image,
   /*
     Convert newlines to multiple lines of text.
   */
-  textlist=StringToList(draw_info->text);
+  textlist=StringToStrings(draw_info->text,&count);
   if (textlist == (char **) NULL)
     return(MagickFalse);
   annotate_info->render=MagickFalse;
@@ -738,21 +749,42 @@ MagickExport MagickBooleanType GetMultilineTypeMetrics(Image *image,
   annotate_info->text=textlist[0];
   status=GetTypeMetrics(image,annotate_info,&extent);
   *metrics=extent;
-  for (i=1; textlist[i] != (char *) NULL; i++)
-  {
-    annotate_info->text=textlist[i];
-    status=GetTypeMetrics(image,annotate_info,&extent);
-    if (extent.width > metrics->width)
-      *metrics=extent;
-  }
-  metrics->height=(double) (i*(size_t) (metrics->ascent-metrics->descent+0.5)+
-    (i-1)*draw_info->interline_spacing);
+  height=(count*(size_t) (metrics->ascent-metrics->descent+
+    0.5)+(count-1)*draw_info->interline_spacing);
+  size=(MagickSizeType) fabs(height);
+  if (AcquireMagickResource(HeightResource,size) == MagickFalse)
+    {
+      (void) ThrowMagickException(&image->exception,GetMagickModule(),
+        ImageError,"WidthOrHeightExceedsLimit","`%s'",image->filename);
+      status=MagickFalse;
+    }
+  else
+    {
+      for (i=1; i < (ssize_t) count; i++)
+      {
+        annotate_info->text=textlist[i];
+        status=GetTypeMetrics(image,annotate_info,&extent);
+        if (status == MagickFalse)
+          break;
+        if (extent.width > metrics->width)
+          *metrics=extent;
+        size=(MagickSizeType) fabs(extent.width);
+        if (AcquireMagickResource(WidthResource,size) == MagickFalse)
+          {
+            (void) ThrowMagickException(&image->exception,GetMagickModule(),
+              ImageError,"WidthOrHeightExceedsLimit","`%s'",image->filename);
+            status=MagickFalse;
+            break;
+          }
+      }
+      metrics->height=(double) height;
+    }
   /*
     Relinquish resources.
   */
   annotate_info->text=(char *) NULL;
   annotate_info=DestroyDrawInfo(annotate_info);
-  for (i=0; textlist[i] != (char *) NULL; i++)
+  for (i=0; i < (ssize_t) count; i++)
     textlist[i]=DestroyString(textlist[i]);
   textlist=(char **) RelinquishMagickMemory(textlist);
   return(status);
@@ -911,8 +943,9 @@ static MagickBooleanType RenderType(Image *image,const DrawInfo *draw_info,
   if ((type_info == (const TypeInfo *) NULL) &&
       (draw_info->family != (const char *) NULL))
     {
-      type_info=GetTypeInfoByFamily(draw_info->family,draw_info->style,
-        draw_info->stretch,draw_info->weight,&image->exception);
+      if (strchr(draw_info->family,',') == (char *) NULL)
+        type_info=GetTypeInfoByFamily(draw_info->family,draw_info->style,
+          draw_info->stretch,draw_info->weight,&image->exception);
       if (type_info == (const TypeInfo *) NULL)
         {
           char
@@ -932,7 +965,8 @@ static MagickBooleanType RenderType(Image *image,const DrawInfo *draw_info,
           {
             type_info=GetTypeInfoByFamily(family[i],draw_info->style,
               draw_info->stretch,draw_info->weight,&image->exception);
-            if (type_info != (const TypeInfo *) NULL)
+            if ((type_info != (const TypeInfo *) NULL) &&
+                (LocaleCompare(family[i],type_info->family) == 0))
               break;
           }
           for (i=0; i < (ssize_t) number_families; i++)
@@ -1267,7 +1301,8 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
 
   FT_UInt
     first_glyph_id,
-    last_glyph_id;
+    last_glyph_id,
+    missing_glyph_id;
 
   FT_Vector
     origin;
@@ -1296,6 +1331,7 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
 
   ssize_t
     code,
+    last_character,
     y;
 
   static FT_Outline_Funcs
@@ -1435,9 +1471,9 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
   metrics->bounds.x2=metrics->ascent+metrics->descent;
   metrics->bounds.y2=metrics->ascent+metrics->descent;
   metrics->underline_position=face->underline_position*
-    (metrics->pixels_per_em.x/face->units_per_EM);
+    (metrics->pixels_per_em.x*PerceptibleReciprocal(face->units_per_EM));
   metrics->underline_thickness=face->underline_thickness*
-    (metrics->pixels_per_em.x/face->units_per_EM);
+    (metrics->pixels_per_em.x*PerceptibleReciprocal(face->units_per_EM));
   first_glyph_id=0;
   FT_Get_First_Char(face,&first_glyph_id);
   if ((draw_info->text == (char *) NULL) || (*draw_info->text == '\0') ||
@@ -1461,7 +1497,7 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
   if (draw_info->render == MagickFalse)
     flags=FT_LOAD_NO_BITMAP;
   if (draw_info->text_antialias == MagickFalse)
-    flags|=FT_LOAD_TARGET_MONO;
+    flags|=FT_LOAD_RENDER | FT_LOAD_TARGET_MONO;
   else
     {
 #if defined(FT_LOAD_TARGET_LIGHT)
@@ -1517,7 +1553,9 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
     }
   grapheme=(GraphemeInfo *) NULL;
   length=ComplexTextLayout(image,draw_info,p,strlen(p),face,flags,&grapheme);
+  missing_glyph_id=FT_Get_Char_Index(face,' ');
   code=0;
+  last_character=(ssize_t) length-1;
   for (i=0; i < (ssize_t) length; i++)
   {
     FT_Outline
@@ -1528,7 +1566,7 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
     */
     glyph.id=grapheme[i].index;
     if (glyph.id == 0)
-      glyph.id=FT_Get_Char_Index(face,' ');
+      glyph.id=missing_glyph_id;
     if ((glyph.id != 0) && (last_glyph_id != 0))
       origin.x+=(FT_Pos) (64.0*draw_info->kerning);
     glyph.origin=origin;
@@ -1583,7 +1621,7 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
     bitmap=(FT_BitmapGlyph) glyph.image;
     point.x=offset->x+bitmap->left;
     if (bitmap->bitmap.pixel_mode == ft_pixel_mode_mono)
-      point.x=offset->x+(origin.x >> 6);
+      point.x+=(origin.x/64.0);
     point.y=offset->y-bitmap->top;
     if (draw_info->render != MagickFalse)
       {
@@ -1644,22 +1682,27 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
                 bitmap->bitmap.width,1,exception);
               active=q != (PixelPacket *) NULL ? MagickTrue : MagickFalse;
             }
-          n=y*bitmap->bitmap.pitch-1;
-          for (x=0; x < (ssize_t) bitmap->bitmap.width; x++)
+          n=y*bitmap->bitmap.pitch;
+          for (x=0; x < (ssize_t) bitmap->bitmap.width; x++, n++)
           {
-            n++;
-            x_offset++;
+            x_offset=(ssize_t) ceil(point.x+x-0.5);
             if ((x_offset < 0) || (x_offset >= (ssize_t) image->columns))
               {
                 if (q != (PixelPacket *) NULL)
                   q++;
                 continue;
               }
-            if (bitmap->bitmap.pixel_mode != ft_pixel_mode_mono)
-              fill_opacity=(MagickRealType) (p[n])/(bitmap->bitmap.num_grays-1);
-            else
-              fill_opacity=((p[(x >> 3)+y*bitmap->bitmap.pitch] &
-                (1 << (~x & 0x07)))) == 0 ? 0.0 : 1.0;
+            fill_opacity=1.0;
+            if (bitmap->bitmap.buffer != (unsigned char *) NULL)
+              {
+                if (bitmap->bitmap.pixel_mode == ft_pixel_mode_grays)
+                  fill_opacity=(MagickRealType) (p[n])/(
+                    bitmap->bitmap.num_grays-1);
+                else
+                  if (bitmap->bitmap.pixel_mode == ft_pixel_mode_mono)
+                    fill_opacity=((p[(x >> 3)+y*bitmap->bitmap.pitch] &
+                      (1 << (~x & 0x07)))) == 0 ? 0.0 : 1.0;
+              }
             if (draw_info->text_antialias == MagickFalse)
               fill_opacity=fill_opacity >= 0.5 ? 1.0 : 0.0;
             if (active == MagickFalse)
@@ -1719,7 +1762,10 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
         (IsUTFSpace(code) == MagickFalse))
       origin.x+=(FT_Pos) (64.0*draw_info->interword_spacing);
     else
-      origin.x+=(FT_Pos) grapheme[i].x_advance;
+      if (i == last_character)
+        origin.x+=MagickMax((FT_Pos) grapheme[i].x_advance,bounds.xMax);
+      else
+        origin.x+=(FT_Pos) grapheme[i].x_advance;
     metrics->origin.x=(double) origin.x;
     metrics->origin.y=(double) origin.y;
     if (metrics->origin.x > metrics->width)
